# -*- coding: utf-8 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
Manage backward compatibility for astk < 1.6.0

- change_argv :
    Example :
        astk  1.6.0  :  as_run --info
        astk <1.6.0  :  as_serv as_info
"""

import sys
import re
import os.path as osp
from warnings import warn, simplefilter

from asrun.common.i18n import _


# DeprecationWarning are ignored in python2.7 by default
simplefilter('default')

dict_serv = {
    'as_info'         : '--info',
    'as_edit'         : '--edit',
    'as_actu'         : '--actu',
    'as_tail'         : '--tail',
    'as_del'          : '--del',
    'as_rex_creer'    : '--create_issue',
    'insert_in_db'    : '--insert_in_db',
    'as_exec'         : '--serv',
    'as_exec_special' : '--serv',
    'as_mail'         : '--sendmail',
}


def chg_as_mail(oldargv):
    """Changes for 'as_serv as_mail email_address file'.
    """
    argv = oldargv[:]
    if len(argv) > 2:
        argv[2] = '--report_to=%s' % argv[2]
    return argv


def change_argv(oldargv):
    """Change command line arguments
    """
    from asrun.logger import log
    from asrun.parser import get_option_value
    # replace first argument (service) by corresponding option
    argv = oldargv[:]
    arg1 = ''
    if len(argv[1:]) > 0:
        arg1 = argv[1]
    if dict_serv.get(arg1) is not None:
        argv[1] = dict_serv[arg1]
    elif arg1 in ('as_exec', 'as_exec_special'):
        pass
    elif arg1 in ('tool_stanley', 'as_rex_consult', 'as_rex_modfic', 'as_rex_suppr'):
        log.fatal(_(u"The service (%s) is not available any more. Update your ASTK client.") % arg1)
        sys.exit(2)

    if arg1 == 'as_mail':
        argv = chg_as_mail(argv)

    # change old style options (-KILL/-USR1)
    dict_change = {
        '-KILL' : '--signal=KILL',
        '-USR1' : '--signal=USR1',
    }
    ind = range(len(argv))
    d = dict(zip(argv, ind))
    for old, new in dict_change.items():
        if d.get(old) is not None:
            argv[d[old]] = new

    #pylint: disable-msg=E1103
    # rcdir=xxx previously accepted a suffix, cause of confusion
    rcdir = get_option_value(argv, "--rcdir")
    rcdir0 = rcdir
    if rcdir is not None and not rcdir.startswith(".astkrc") and osp.abspath(rcdir) != rcdir:
        warn("rcdir option requires an absolute path or something similar to '.astkrc_xxx'", 
             DeprecationWarning, stacklevel=2)
        rcdir = ".astkrc_%s" % rcdir
        for i, v in enumerate(argv):
            if v.find("--rcdir=") > -1:
                argv[i] = "--rcdir=%s" % rcdir
                log.debug("rcdir argument %s changed to %s", rcdir0, argv[i])

    # warn
    if argv != oldargv:
        s_old = ' '.join(oldargv)
        argv2 = argv[:]
        if argv2[0].endswith('as_serv'):
            argv2[0] = re.sub('as_serv$', 'as_run', argv2[0])
        s_new = ' '.join(argv2)
    
        msg = """command line %s is deprecated, please use %s instead.""" % (s_old, s_new)
        warn(msg, DeprecationWarning, stacklevel=2)
    return argv


# 1.7.12 > 1.8.0
def read_rcfile(*args, **kwargs):
    warn('read_rcfile moved to asrun.common.rcfile', DeprecationWarning, stacklevel=2)
    from asrun.common.rcfile import read_rcfile
    return read_rcfile(*args, **kwargs)

def parse_config(*args, **kwargs):
    warn('parse_config moved to asrun.common.rcfile', DeprecationWarning, stacklevel=2)
    from asrun.common.rcfile import parse_config
    return parse_config(*args, **kwargs)

# 1.8.0 > 1.8.1
def get_timeout(prof):
    warn('get_timeout is now a method of ASTER_PROFIL object', DeprecationWarning,
        stacklevel=2)
    return prof.get_timeout()

def add_param(prof, dict_para):
    warn('add_param is now a method of ASTER_PROFIL object named add_param_from_dict',
        DeprecationWarning, stacklevel=2)
    return prof.add_param_from_dict(dict_para)

def get_hostrc(run, prof):
    warn('get_hostrc moved to asrun.repart', DeprecationWarning, stacklevel=2)
    from asrun.repart import get_hostrc
    return get_hostrc(run, prof)

# 1.8.3 > 1.8.4
def bwc_edit_args(args):
    if len(args) > 4:
        warn("'as_run --edit' does not support display argument anymore",
            DeprecationWarning, stacklevel=2)
    return args[:4]

